import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

import { setAfterMark, render, setMarkTagName, setDefaultClass } from './core';
var pageHashPrefix = 'qoxop_mark_';
var config = {
  pageHashPrefix: pageHashPrefix,
  pageHash: function pageHash(href) {
    return pageHashPrefix + href.replace(location.origin, '');
  },
  getPageInfo: function getPageInfo() {
    return {
      title: document.title
    };
  }
};
var MarkStore = {};
/**
 *  清除页面副作用
 * @param marks 
 */

function clearDomSideEffect(marks) {
  marks.reverse().forEach(function (item) {
    var domArray = Array.from(document.getElementsByClassName(item.id) || []);
    domArray.forEach(function (elem) {
      return elem.replaceWith(elem.firstChild);
    });
  });
  document.normalize();
}
/**
 * 获取当前页面的所有标记
 */


function getCurrentPageMarks() {
  var pageHash = config.pageHash,
      getPageInfo = config.getPageInfo;
  var hash = pageHash(window.location.href);

  if (!MarkStore[hash]) {
    MarkStore[hash] = JSON.parse(window.localStorage.getItem(hash)) || {
      marks: [],
      pageInfo: getPageInfo()
    };
  }

  return MarkStore[hash];
}
/**
 * 设置配置项
 * @param options 
 */


function setConfig() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var obj = Object.assign({
    markTagName: 'marks',
    defaultClassName: 'qoxop_highlight'
  }, options);
  Object.keys(obj).forEach(function (key) {
    var value = obj[key];

    if (key === 'markTagName') {
      setMarkTagName(value);
    } else if (key === 'defaultClassName') {
      setDefaultClass(value);
    } else {
      config[key] = value;
    }
  });
  pageHashPrefix = config.pageHashPrefix;
  return config;
}
/**
 * 渲染当前页的所有标记
 */


function renderAll() {
  document.normalize();
  var allmarks = getCurrentPageMarks();

  if (!allmarks || !allmarks.marks || allmarks.marks.length === 0) {
    return true;
  }

  var elems = document.getElementsByClassName(allmarks.marks[0].id);

  if (elems && elems.length > 0) {
    return true;
  }

  if (!allmarks.marks.every(function (item) {
    return render(item);
  })) {
    removeAll({
      domain: false,
      retainTexts: true
    });
  }

  return true;
}
/**
 * 移除标记
 * @param id 
 * @param allRmHandler 
 */


function remove(id, allRmHandler) {
  var allMarks = getCurrentPageMarks();
  allMarks.marks.some(function (item) {
    if (id === item.id && !item.unused) {
      var elems = document.getElementsByClassName(id); // 删除class

      var elemArr = Array.prototype.slice.call(elems);
      elemArr.forEach(function (elem) {
        return elem.setAttribute('class', id);
      });
      item.unused = true;
      return true;
    }
  });

  if (allMarks.marks.every(function (item) {
    return item.unused;
  })) {
    if (allRmHandler) {
      allRmHandler(function () {
        allMarks.marks = [];
        removeAll({
          domain: false,
          retainTexts: false
        });
      }, allMarks.marks);
    } else {
      removeAll({
        domain: false,
        retainTexts: false
      });
    }
  } else {
    var pageHash = config.pageHash;
    var hash = pageHash(window.location.href);
    window.localStorage.setItem(hash, JSON.stringify(allMarks));
  }
}

/**
 * 标记文本查询
 */
var query = {
  marks: function marks() {
    return Object.keys(localStorage).filter(function (key) {
      return key.indexOf(pageHashPrefix) === 0;
    }).reduce(function (dateset, key) {
      return _objectSpread({}, dateset, _defineProperty({}, key, JSON.parse(localStorage[key])));
    }, {});
  },
  texts: function texts(includeHistory) {
    var historys = {};

    if (includeHistory) {
      historys = Object.keys(localStorage).filter(function (key) {
        return key.indexOf("history_".concat(pageHashPrefix, "_")) === 0;
      }).reduce(function (dateset, key) {
        return _objectSpread({}, dateset, _defineProperty({}, key, JSON.parse(localStorage[key])));
      }, {});
    }

    var marks = query.marks();
    return Object.keys(marks).reduce(function (dataset, key) {
      return _objectSpread({}, dataset, _defineProperty({}, key, marks[key].marks.map(function (m) {
        return m.text;
      })));
    }, historys);
  }
};

/**
 * 清除所有标记
 * @param params 
 */
function removeAll() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _domain$retainTexts$o = _objectSpread({
    domain: false,
    retainTexts: false
  }, options),
      domain = _domain$retainTexts$o.domain,
      retainTexts = _domain$retainTexts$o.retainTexts;

  var pageHash = config.pageHash(window.location.href);
  var markSet = [];
  Object.keys(localStorage).filter(function (key) {
    if (domain === true) {
      return key.indexOf(pageHashPrefix) === 0;
    } else {
      return key === pageHash;
    }
  }).forEach(function (key) {
    try {
      var _marks = JSON.parse(localStorage.getItem(key) || '{}').marks;
      markSet = markSet.concat(_marks);

      if (retainTexts) {
        // 保存文本备份
        var historys = JSON.parse(localStorage.getItem("history_".concat(pageHashPrefix)) || '[]');
        localStorage.setItem("history_".concat(pageHashPrefix), JSON.stringify(historys.concat(_marks.map(function (item) {
          return item.text;
        }))));
      }
    } catch (error) {
      console.error(error);
    }

    localStorage.removeItem(key);
    delete MarkStore[key];
  }); // 清除页面副作用的方式

  clearDomSideEffect(markSet);
}
/**
 * 标记后存储
 */


setAfterMark(function (mi) {
  var pageHash = config.pageHash,
      getPageInfo = config.getPageInfo;
  var hash = pageHash(mi.href);
  var storeMi = mi;
  delete storeMi.textNodes.all;
  delete storeMi.container.elem;

  if (!MarkStore[hash]) {
    MarkStore[hash] = {
      marks: [storeMi],
      pageInfo: getPageInfo()
    };
  } else {
    MarkStore[hash].marks.push(storeMi);
  }

  window.localStorage.setItem(hash, JSON.stringify(MarkStore[hash]));
});
export { remove, setConfig, removeAll, renderAll, query };