"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.countSelectionInfo = countSelectionInfo;
exports.render = render;
exports["default"] = _default;
exports.setDefaultClass = exports.setMarkTagName = exports.setAfterMark = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var CustomTagName = 'marks';
var HighLightClass = 'qoxop_highlight';
/**
 * 获取首尾节点的字符串，用于校验标记是否作废
 * @param start
 * @param end 
 */

function countStartEndChunk(start, end) {
  return "".concat(start.nodeValue).concat(end.nodeValue).trim();
}
/**
 * 查询node下的所有文本节点
 * @param {Node} node 
 */


function QueryAllTextNodes(node) {
  var texts = [];

  if (node) {
    if (document.createNodeIterator) {
      var textIterator = document.createNodeIterator(node, NodeFilter.SHOW_TEXT);
      var textNode = textIterator.nextNode();

      while (textNode) {
        texts.push(textNode);
        textNode = textIterator.nextNode();
      }

      return texts;
    } else {
      for (var i = 0; i < node.childNodes.length; i++) {
        if (node.childNodes[i].hasChildNodes()) {
          texts.push.apply(texts, (0, _toConsumableArray2["default"])(QueryAllTextNodes(node.childNodes[i])));
        } else if (node.childNodes[i].nodeType === Node.TEXT_NODE) {
          texts.push(node.childNodes[i]);
        }
      }
    }
  }

  return texts;
}
/**
 * 计算选中文本在文档中的位置
 */


function countSelectionInfo() {
  var selection = window.getSelection();

  if (!selection.isCollapsed) {
    var range = selection.getRangeAt(0);
    var startOffset = range.startOffset,
        endOffset = range.endOffset;
    var selectionInfo = {
      id: window.btoa("".concat(Date.now() * Math.random()).concat((Math.random() * 100).toFixed(2))),
      container: {
        index: -1,
        tagname: ''
      },
      textNodes: {
        start: null,
        end: null,
        startEndChunk: ''
      },
      href: window.location.href,
      time: Date.now(),
      text: selection.toString()
    }; // 寻找容器元素

    var containerElem = range.commonAncestorContainer;

    while (true) {
      if (containerElem.nodeType === Node.ELEMENT_NODE || !containerElem) {
        break;
      }

      containerElem = containerElem.parentElement;
    } // 记住node位置            


    var allTextNodes = QueryAllTextNodes(containerElem);

    for (var i = 0; i < allTextNodes.length; i++) {
      if (allTextNodes[i] === range.startContainer) {
        selectionInfo.textNodes.start = {
          index: i,
          split: startOffset
        };
      }

      if (allTextNodes[i] === range.endContainer) {
        selectionInfo.textNodes.end = {
          index: i,
          split: endOffset
        };
        break;
      }
    }

    selectionInfo.textNodes.all = allTextNodes;
    selectionInfo.textNodes.startEndChunk = countStartEndChunk(range.startContainer, range.endContainer); // 记住元素位置

    var tags = document.getElementsByTagName(containerElem.nodeName);

    for (var _i = 0; _i < tags.length; _i++) {
      if (tags.item(_i) === containerElem) {
        selectionInfo.container.index = _i;
        selectionInfo.container.tagname = containerElem.nodeName;
        selectionInfo.container.elem = containerElem;
        break;
      }
    }

    selection.removeAllRanges();
    return selectionInfo;
  }

  return false;
}
/**
 * 渲染标记文本
 * @param MarkInfo 
 */


function render(mark) {
  var _mark$container = mark.container,
      tagIndex = _mark$container.index,
      tagname = _mark$container.tagname,
      _mark$textNodes = mark.textNodes,
      start = _mark$textNodes.start,
      end = _mark$textNodes.end;
  var elem = mark.container.elem,
      allTexts = mark.textNodes.all;
  var Wrp = document.createElement(CustomTagName);
  var className = "".concat(HighLightClass, " ").concat(mark.className ? mark.className : '');
  Wrp.setAttribute('class', "".concat(mark.unused ? '' : className, " ").concat(mark.id));
  Wrp.setAttribute('mark_id', mark.id);

  if (mark.style) {
    Object.keys(mark.style).forEach(function (key) {
      // @ts-ignore
      Wrp.style[key] = mark.style[key];
    });
  }

  if (mark.meta) {
    Wrp.setAttribute('meta_data', JSON.stringify(mark.meta));
  }

  if (!allTexts || allTexts.length < 1) {
    if (!(elem instanceof Element)) {
      var elems = document.getElementsByTagName(tagname);
      elem = elems.item(tagIndex);
    }

    allTexts = QueryAllTextNodes(elem); // check chunk

    if (allTexts && allTexts[start.index] && allTexts[end.index]) {
      var startEndChunk = countStartEndChunk(allTexts[start.index], allTexts[end.index]);

      if (mark.textNodes.startEndChunk !== startEndChunk) {
        return false;
      }
    } else {
      return false;
    }
  }

  var WrpFn = function WrpFn(text) {
    var wrp = Wrp.cloneNode();
    wrp.innerHTML = text.nodeValue;
    text.parentNode.replaceChild(wrp, text);
  };

  try {
    if (start.index === end.index) {
      var nextText = allTexts[start.index].splitText(start.split).splitText(end.split - start.split);
      WrpFn(nextText.previousSibling);
      return true;
    }

    for (var i = start.index + 1; i < end.index; i++) {
      WrpFn(allTexts[i]);
    }

    var startText = allTexts[start.index].splitText(start.split);
    var endText = allTexts[end.index].splitText(end.split).previousSibling;
    WrpFn(startText);
    WrpFn(endText);
    return true;
  } catch (error) {
    console.error(error);
    return false;
  }
}

var _ref = function () {
  var afns = [];
  return {
    setAfterMark: function setAfterMark(fn) {
      afns.push(fn);
    },
    runAfters: function runAfters(mi) {
      afns.reduce(function (old, fn) {
        return fn(mi, old);
      }, null);
    }
  };
}(),
    setAfterMark = _ref.setAfterMark,
    runAfters = _ref.runAfters;

exports.setAfterMark = setAfterMark;

var setMarkTagName = function setMarkTagName(tagname) {
  return CustomTagName = tagname;
};

exports.setMarkTagName = setMarkTagName;

var setDefaultClass = function setDefaultClass(className) {
  return HighLightClass = className;
};

exports.setDefaultClass = setDefaultClass;

function _default() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var callback = arguments.length > 1 ? arguments[1] : undefined;
  var info = countSelectionInfo();
  var success = false;

  if (info !== false) {
    var _mi = _objectSpread({}, info, {}, options);

    success = render(_mi);

    if (success) {
      runAfters(_mi);
    }

    if (typeof callback === 'function') {
      callback(_mi, success);
    }
  }

  return success;
}